package com.dragon.car.utils;

import com.dragon.common.core.constant.SecurityConstants;
import com.dragon.common.core.domain.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Map;

/**
 * @author : Administrator
 * @Description : aop日志
 */
@Aspect
@Component
@Slf4j
public class LogRecordAspect {
	/** 定义切面表达式 */
	private static final String POINT = "execution(* com.dragon.car.server.controller..*.*(..))";
	/** 记录每次请求的开始时间 */
	private static final ThreadLocal<Long> START_TIME = new ThreadLocal<> ();
	/** 记录每次请求的序列号 */
	private static final ThreadLocal<String> SEQUENCE = new ThreadLocal<> ();
	@Resource
	HttpServletRequest request;
	
	@Pointcut (POINT)
	public void executeService () {
	
	}
	
	@Before ("executeService()")
	public void beFore (JoinPoint pjp) {
		
		//请求信息
		RequestAttributes req = RequestContextHolder.getRequestAttributes ();
		ServletRequestAttributes reqSra = (ServletRequestAttributes) req;
		HttpServletRequest request = null;
		if (reqSra != null) {
			request = reqSra.getRequest ();
		}
		String header = null;
		if (request != null) {
			header = request.getHeader ("X-Forwarded-For");
		}
		if (header == null || header.isEmpty () || "unknown".equalsIgnoreCase (header)) {
			
			if (request != null) {
				header = request.getRemoteAddr ();
			}
		}
		else {
			
			int i = header.indexOf (",");
			if (i != -1) {
				header = header.substring (0, i);
			}
		}
		/*ip地址是*/
		String ipAddr = null;
		if (header != null) {
			ipAddr = header.trim ();
		}
		Enumeration<String> headerNames = null;
		if (request != null) {
			headerNames = request.getHeaderNames ();
		}
		StringBuilder stringBuilder = new StringBuilder ();
		if (headerNames != null) {
			while (headerNames.hasMoreElements ()) {
				String s = headerNames.nextElement ();
				stringBuilder.append (s)
				             .append (":")
				             .append (request.getHeader (s))
				             .append ("|");
			}
		}
		/*解析参数*/
		Object[] pjpArgs = pjp.getArgs ();
		Map<String, String[]> parameterMap = null;
		if (request != null) {
			parameterMap = request.getParameterMap ();
		}
		StringBuilder builder = new StringBuilder ();
		if (parameterMap != null) {
			parameterMap.forEach ((k, v) -> builder
					                                .append (k)
					                                .append (":")
					                                .append (Arrays.deepToString (v))
					                                .append ("|")
			                     );
		}
		/*每次请求的序列号,避免方便匹配同一个请求以及对应的响应*/
		
		SEQUENCE.set (getRandom ());
		if (request != null) {
			log.info (
					"trace log begin --> sequence::{},userName::{},RealIp::{},User-Agent::{},requestUri::{}" + ",header::{},requestMethod::{},request::{}, args::{}",
					SEQUENCE.get (),
					SecurityConstants.DETAILS_USERNAME,
					ipAddr,
					request.getHeader ("User-Agent"),
					request.getRequestURI (),
					builder.toString (),
					request.getMethod (),
					stringBuilder.toString (),
					Arrays.deepToString (pjpArgs)
			         );
		}
		
		START_TIME.set (System.currentTimeMillis ());
		
	}
	
	public String getRandom () {
		
		return RandomStringUtils.random (8, true, true);
	}
	
	@AfterReturning (pointcut = "executeService()", returning = "result")
	public void after (Object result) {
		try {
			String res = "not standard response";
			//自定义返回标准类
			if (result instanceof Result) {
				res = result.toString ();
			}
			log.info (
					"""
							trace log end -->sequence::{}, used::{}ms, responseBody::{}
							""",
					SEQUENCE.get (), (System.currentTimeMillis () - START_TIME.get ()),
					res
			         );
			
		}
		catch (Exception e) {
			throw new RuntimeException (e);
		}
		finally {
			START_TIME.remove ();
			SEQUENCE.remove ();
		}
	}
}
