package com.dragon.car.service.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dragon.car.domain.req.RequestCar;
import com.dragon.car.domain.resp.ResquestFaultCode;
import com.dragon.car.service.mapper.CarManageMapper;
import com.dragon.car.service.service.CarManageService;
import com.dragon.common.core.domain.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author : Administrator
 * @description : 车辆业务逻辑层
 */
@Service
@Log4j2
public class CarManageServiceImpl implements CarManageService {
	
	@Resource
	CarManageMapper carMapper;
	
	
	/**
	 * 初始化车辆
	 *
	 * @return List<CarInit> 企业管理的车-列表
	 */
	@Override
	public Result<List<RequestCar>> carInit () {
		LambdaQueryWrapper<RequestCar> aclqw = new LambdaQueryWrapper<> ();
		
		List<RequestCar> carInits = carMapper.selectList (aclqw);
		
		log.info ("信息是:{}", carInits.stream ().toArray ());
		return Result.success (carInits);
	}
	
	/**
	 * 车俩添加方法
	 *
	 * @param request
	 *
	 * @return 是否成功 i=1 成功 i=0 失败
	 */
	@Override
	public Result carInsert (RequestCar request) {
		
		
		int i = carMapper.insert (request);
		
		return Result.success (i > 0 ? "成功" : "失败");
	}
	
	/**
	 * 车辆修改方法
	 *
	 * @param request
	 *
	 * @return 是否成功 i=1 成功 i=0 失败
	 */
	@Override
	public Result carUpdate (RequestCar request) {
		LambdaUpdateWrapper luw = new LambdaUpdateWrapper ();
		int i = carMapper.update (request, luw);
		return Result.success (i > 0 ? "成功" : "失败");
	}
	
	/**
	 * 车辆删除
	 *
	 * @param request
	 *
	 * @return 是否成功 i=1 成功 i=0 失败
	 */
	@Override
	public Result carDelect (RequestCar request) {
		int i = carMapper.deleteById (request.getCarId ());
		
		return Result.success (i > 0 ? "成功" : "失败");
	}
	
	/**
	 * @return
	 */
	@Override
	public Result<List<ResquestFaultCode>> faultCodeList () {
		return null;
	}
	
	
}
